@@ -1,4 +1,4 @@
 """Specific verbs/funcs from this package"""
 
-from .verbs import get, flatten, drop_index
+from .verbs import get, flatten
 from .funcs import itemgetter
