@@ -7,7 +7,8 @@ from pandas import DataFrame
 from pipda import register_verb
 
 from ..core.contexts import Context
-from ..core.utils import arg_match, vars_select, reconstruct_tibble
+from ..core.utils import arg_match, vars_select
+from ..core.tibble import reconstruct_tibble
 
 
 @register_verb(DataFrame, context=Context.SELECT)
@@ -15,7 +16,6 @@ def drop_na(
     _data: DataFrame,
     *columns: str,
     how_: str = "any",
-    base0_: bool = None,
 ) -> DataFrame:
     """Drop rows containing missing values
 
@@ -28,8 +28,6 @@ def drop_na(
             - all: All columns of `columns` to be `NA`s
             - any: Any columns of `columns` to be `NA`s
             (tidyr doesn't support this argument)
-        base0_: Whether `*columns` are 0-based if given by indexes
-            If not provided, will use `datar.base.get_option('index.base.0')`
 
     Returns:
         Dataframe with rows with NAs dropped and indexes dropped
@@ -37,10 +35,10 @@ def drop_na(
     arg_match(how_, "how_", ["any", "all"])
     all_columns = _data.columns
     if columns:
-        columns = vars_select(all_columns, *columns, base0=base0_)
+        columns = vars_select(all_columns, *columns)
         columns = all_columns[columns]
         out = _data.dropna(subset=columns, how=how_).reset_index(drop=True)
     else:
         out = _data.dropna(how=how_).reset_index(drop=True)
 
-    return reconstruct_tibble(_data, out, keep_rowwise=True)
+    return reconstruct_tibble(_data, out)
